@@ -1,12 +1,23 @@
 #include "gpu.h"
 #include <inttypes.h>
+#include <memory>
+#include <functional>
+#include <thread>
+#include <cstring>
+#include <spdlog/spdlog.h>
 #include "nvctrl.h"
+#include "timing.hpp"
 #ifdef HAVE_NVML
 #include "nvidia_info.h"
 #endif
 
-struct gpuInfo gpu_info;
+#include "amdgpu.h"
+
+using namespace std::chrono_literals;
+
+struct gpuInfo gpu_info {};
 amdgpu_files amdgpu {};
+decltype(&getAmdGpuInfo) getAmdGpuInfo_actual = nullptr;
 
 bool checkNvidia(const char *pci_dev){
     bool nvSuccess = false;
@@ -35,6 +46,9 @@ void getNvidiaGpuInfo(){
         gpu_info.MemClock = nvidiaMemClock;
         gpu_info.powerUsage = nvidiaPowerUsage / 1000;
         gpu_info.memoryTotal = nvidiaMemory.total / (1024.f * 1024.f * 1024.f);
+        gpu_info.is_temp_throttled = (nvml_throttle_reasons & 0x0000000000000060LL) != 0;
+        gpu_info.is_power_throttled = (nvml_throttle_reasons & 0x000000000000008CLL) != 0;
+        gpu_info.is_other_throttled = (nvml_throttle_reasons & 0x0000000000000112LL) != 0;
         return;
     }
 #endif
@@ -57,25 +71,53 @@ nvapi_util();
 }
 
 void getAmdGpuInfo(){
-    if (amdgpu.busy) {
-        rewind(amdgpu.busy);
-        fflush(amdgpu.busy);
-        int value = 0;
-        if (fscanf(amdgpu.busy, "%d", &value) != 1)
-            value = 0;
-        gpu_info.load = value;
-    }
-
-    if (amdgpu.temp) {
-        rewind(amdgpu.temp);
-        fflush(amdgpu.temp);
-        int value = 0;
-        if (fscanf(amdgpu.temp, "%d", &value) != 1)
-            value = 0;
-        gpu_info.temp = value / 1000;
-    }
-
     int64_t value = 0;
+    if (metrics_path.empty()){
+        if (amdgpu.busy) {
+            rewind(amdgpu.busy);
+            fflush(amdgpu.busy);
+            int value = 0;
+            if (fscanf(amdgpu.busy, "%d", &value) != 1)
+                value = 0;
+            gpu_info.load = value;
+        }
+
+        if (amdgpu.temp) {
+            rewind(amdgpu.temp);
+            fflush(amdgpu.temp);
+            int value = 0;
+            if (fscanf(amdgpu.temp, "%d", &value) != 1)
+                value = 0;
+            gpu_info.temp = value / 1000;
+        }
+
+        if (amdgpu.core_clock) {
+            rewind(amdgpu.core_clock);
+            fflush(amdgpu.core_clock);
+            if (fscanf(amdgpu.core_clock, "%" PRId64, &value) != 1)
+                value = 0;
+
+            gpu_info.CoreClock = value / 1000000;
+        }
+
+        if (amdgpu.memory_clock) {
+            rewind(amdgpu.memory_clock);
+            fflush(amdgpu.memory_clock);
+            if (fscanf(amdgpu.memory_clock, "%" PRId64, &value) != 1)
+                value = 0;
+
+            gpu_info.MemClock = value / 1000000;
+        }
+
+        if (amdgpu.power_usage) {
+            rewind(amdgpu.power_usage);
+            fflush(amdgpu.power_usage);
+            if (fscanf(amdgpu.power_usage, "%" PRId64, &value) != 1)
+                value = 0;
+
+            gpu_info.powerUsage = value / 1000000;
+        }
+    }
 
     if (amdgpu.vram_total) {
         rewind(amdgpu.vram_total);
@@ -92,31 +134,4 @@ void getAmdGpuInfo(){
             value = 0;
         gpu_info.memoryUsed = float(value) / (1024 * 1024 * 1024);
     }
-
-    if (amdgpu.core_clock) {
-        rewind(amdgpu.core_clock);
-        fflush(amdgpu.core_clock);
-        if (fscanf(amdgpu.core_clock, "%" PRId64, &value) != 1)
-            value = 0;
-
-        gpu_info.CoreClock = value / 1000000;
-    }
-
-    if (amdgpu.memory_clock) {
-        rewind(amdgpu.memory_clock);
-        fflush(amdgpu.memory_clock);
-        if (fscanf(amdgpu.memory_clock, "%" PRId64, &value) != 1)
-            value = 0;
-
-        gpu_info.MemClock = value / 1000000;
-    }
-
-    if (amdgpu.power_usage) {
-        rewind(amdgpu.power_usage);
-        fflush(amdgpu.power_usage);
-        if (fscanf(amdgpu.power_usage, "%" PRId64, &value) != 1)
-            value = 0;
-
-        gpu_info.powerUsage = value / 1000000;
-    }
 }
