@@ -2,14 +2,16 @@
 #ifndef MANGOHUD_GPU_H
 #define MANGOHUD_GPU_H
 
-#include <stdio.h>
+#include <cstdio>
+#include <cstdint>
 
 struct amdgpu_files
 {
-    FILE *busy;
-    FILE *temp;
     FILE *vram_total;
     FILE *vram_used;
+    /* The following can be NULL, in that case we're using the gpu_metrics node */
+    FILE *busy;
+    FILE *temp;
     FILE *core_clock;
     FILE *memory_clock;
     FILE *power_usage;
@@ -24,13 +26,20 @@ struct gpuInfo{
     float memoryTotal;
     int MemClock;
     int CoreClock;
-    int powerUsage;
+    float powerUsage;
+    float apu_cpu_power;
+    int apu_cpu_temp;
+    bool is_power_throttled;
+    bool is_current_throttled;
+    bool is_temp_throttled;
+    bool is_other_throttled;
 };
 
 extern struct gpuInfo gpu_info;
 
 void getNvidiaGpuInfo(void);
 void getAmdGpuInfo(void);
+extern decltype(&getAmdGpuInfo) getAmdGpuInfo_actual;
 bool checkNvidia(const char *pci_dev);
 extern void nvapi_util();
 extern bool checkNVAPI();
