@@ -1,12 +1,26 @@
+#include <spdlog/spdlog.h>
 #include <algorithm>
+#include <functional>
+#include <sstream>
 #include <cmath>
+#include "overlay.h"
+#include "overlay_params.h"
 #include "hud_elements.h"
+#include "logging.h"
+#include "battery.h"
+#include "gamepad.h"
 #include "cpu.h"
+#include "gpu.h"
 #include "memory.h"
+#include "iostats.h"
 #include "mesa/util/macros.h"
 #include "string_utils.h"
+#include "app/mangoapp.h"
 #include <IconsForkAwesome.h>
 
+#define CHAR_CELSIUS    "\xe2\x84\x83"
+#define CHAR_FAHRENHEIT "\xe2\x84\x89"
+
 // Cut from https://github.com/ocornut/imgui/pull/2943
 // Probably move to ImGui
 float SRGBToLinear(float in)
@@ -45,7 +59,21 @@ ImVec4 LinearToSRGB(ImVec4 col)
     return col;
 }
 
-void HudElements::convert_colors(struct overlay_params& params)
+template<typename T, typename R = float>
+R format_units(T value, const char*& unit)
+{
+    static const char* const units[] = {"B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB"};
+    size_t u = 0;
+    R out_value = value;
+    while (out_value > 1023 && u < ARRAY_SIZE(units)) {
+        out_value /= 1024;
+        ++u;
+    }
+    unit = units[u];
+    return out_value;
+}
+
+void HudElements::convert_colors(const struct overlay_params& params)
 {
     HUDElements.colors.update = false;
     auto convert = [](unsigned color) -> ImVec4 {
@@ -86,7 +114,7 @@ void HudElements::convert_colors(struct overlay_params& params)
     style.WindowRounding = params.round_corners;
 }
 
-void HudElements::convert_colors(bool do_conv, struct overlay_params& params)
+void HudElements::convert_colors(bool do_conv, const struct overlay_params& params)
 {
     HUDElements.colors.convert = do_conv;
     convert_colors(params);
@@ -157,7 +185,11 @@ void HudElements::gpu_stats(){
         }
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_power]) {
             ImGui::TableNextColumn();
-            right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu_info.powerUsage);
+#ifdef MANGOAPP
+            right_aligned_text(text_color, HUDElements.ralign_width, "%.1f", gpu_info.powerUsage);
+#else
+            right_aligned_text(text_color, HUDElements.ralign_width, "%.0f", gpu_info.powerUsage);
+#endif
             ImGui::SameLine(0, 1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
             ImGui::Text("W");
@@ -218,7 +250,11 @@ void HudElements::cpu_stats(){
         }
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_cpu_power]){
             ImGui::TableNextColumn();
-            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", cpuStats.GetCPUDataTotal().power);
+        #ifdef MANGOAPP
+            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", cpuStats.GetCPUDataTotal().power);
+        #else
+            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.0f", cpuStats.GetCPUDataTotal().power);
+        #endif
             ImGui::SameLine(0, 1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
             ImGui::Text("W");
@@ -229,14 +265,13 @@ void HudElements::cpu_stats(){
 
 void HudElements::core_load(){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_core_load]){
-         int i = 0;
          for (const CPUData &cpuData : cpuStats.GetCPUData())
          {
             ImGui::TableNextRow(); ImGui::TableNextColumn();
             ImGui::TextColored(HUDElements.colors.cpu, "CPU");
             ImGui::SameLine(0, 1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
-            ImGui::TextColored(HUDElements.colors.cpu,"%i", i);
+            ImGui::TextColored(HUDElements.colors.cpu,"%i", cpuData.cpu_id);
             ImGui::PopFont();
             ImGui::TableNextColumn();
             auto text_color = HUDElements.colors.text;
@@ -266,11 +301,12 @@ void HudElements::core_load(){
             ImGui::PushFont(HUDElements.sw_stats->font1);
             ImGui::Text("MHz");
             ImGui::PopFont();
-            i++;
          }
     }
 }
+
 void HudElements::io_stats(){
+#ifndef _WIN32
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_io_read] || HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_io_write]){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_io_read] && !HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_io_write])
@@ -282,7 +318,7 @@ void HudElements::io_stats(){
 
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_io_read]){
             ImGui::TableNextColumn();
-            float val = HUDElements.sw_stats->io.per_second.read;
+            const float val = g_io_stats.per_second.read;
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, val < 100 ? "%.1f" : "%.f", val);
             ImGui::SameLine(0,1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -291,7 +327,7 @@ void HudElements::io_stats(){
         }
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_io_write]){
             ImGui::TableNextColumn();
-            float val = HUDElements.sw_stats->io.per_second.write;
+            const float val = g_io_stats.per_second.write;
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, val < 100 ? "%.1f" : "%.f", val);
             ImGui::SameLine(0,1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -299,6 +335,7 @@ void HudElements::io_stats(){
             ImGui::PopFont();
         }
     }
+#endif
 }
 
 void HudElements::vram(){
@@ -311,6 +348,7 @@ void HudElements::vram(){
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::Text("GiB");
         ImGui::PopFont();
+#ifndef MANGOAPP
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_mem_clock]){
             ImGui::TableNextColumn();
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", gpu_info.MemClock);
@@ -319,10 +357,12 @@ void HudElements::vram(){
             ImGui::Text("MHz");
             ImGui::PopFont();
         }
+#endif
     }
 }
+
 void HudElements::ram(){
-#ifdef __gnu_linux__
+#ifdef __linux__
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_ram]){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         ImGui::TextColored(HUDElements.colors.ram, "RAM");
@@ -332,7 +372,7 @@ void HudElements::ram(){
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::Text("GiB");
         ImGui::PopFont();
-      }
+    }
 
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_ram] && HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_swap]){
         ImGui::TableNextColumn();
@@ -345,8 +385,49 @@ void HudElements::ram(){
 #endif
 }
 
+void HudElements::procmem()
+{
+#ifdef __linux__
+    const char* unit = nullptr;
+    if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem])
+        return;
+
+    ImGui::TableNextRow(); ImGui::TableNextColumn();
+    ImGui::TextColored(HUDElements.colors.ram, "PMEM");
+    ImGui::TableNextColumn();
+    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem.resident, unit));
+    ImGui::SameLine(0,1.0f);
+    ImGui::PushFont(HUDElements.sw_stats->font1);
+    ImGui::Text("%s", unit);
+    ImGui::PopFont();
+
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_shared]){
+        ImGui::TableNextColumn();
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem.shared, unit));
+        ImGui::SameLine(0,1.0f);
+        ImGui::PushFont(HUDElements.sw_stats->font1);
+        ImGui::Text("%s", unit);
+        ImGui::PopFont();
+    }
+
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_shared] && HUDElements.params->table_columns < 4){
+        ImGui::TableNextRow();
+        ImGui::TableNextColumn();
+    }
+
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_virt]){
+        ImGui::TableNextColumn();
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem.virt, unit));
+        ImGui::SameLine(0,1.0f);
+        ImGui::PushFont(HUDElements.sw_stats->font1);
+        ImGui::Text("%s", unit);
+        ImGui::PopFont();
+    }
+#endif
+}
+
 void HudElements::fps(){
-    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_fps]){
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_fps] && !HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_fps_only]){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         ImGui::TextColored(HUDElements.colors.engine, "%s", engines[HUDElements.sw_stats->engine]);
         ImGui::TableNextColumn();
@@ -383,6 +464,28 @@ void HudElements::fps(){
     }
 }
 
+void HudElements::fps_only(){
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_fps_only]){
+        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_fps_color_change]){
+            ImGui::TableNextRow(); ImGui::TableNextColumn();
+            int fps = int(HUDElements.sw_stats->fps);
+            struct LOAD_DATA fps_data = {
+            HUDElements.colors.fps_value_low,
+            HUDElements.colors.fps_value_med,
+            HUDElements.colors.fps_value_high,
+            HUDElements.params->fps_value[0],
+            HUDElements.params->fps_value[1]
+            };
+            auto load_color = change_on_load_temp(fps_data, fps);
+            ImGui::TextColored(load_color, "%.0f", HUDElements.sw_stats->fps);
+        }
+        else {
+            ImGui::TableNextRow(); ImGui::TableNextColumn();
+            ImGui::TextColored(HUDElements.colors.text, "%.0f", HUDElements.sw_stats->fps);
+        }
+    }
+}
+
 void HudElements::gpu_name(){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_name] && !HUDElements.sw_stats->gpuName.empty()){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
@@ -398,7 +501,7 @@ void HudElements::engine_version(){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         ImGui::PushFont(HUDElements.sw_stats->font1);
         if (HUDElements.is_vulkan) {
-            if ((HUDElements.sw_stats->engineName == "DXVK" || HUDElements.sw_stats->engineName == "VKD3D")){
+            if ((HUDElements.sw_stats->engine == EngineTypes::DXVK || HUDElements.sw_stats->engine == EngineTypes::VKD3D)){
                 ImGui::TextColored(HUDElements.colors.engine,
                     "%s/%d.%d.%d", HUDElements.sw_stats->engineVersion.c_str(),
                     HUDElements.sw_stats->version_vk.major,
@@ -459,7 +562,7 @@ void HudElements::frame_timing(){
         for (size_t i = 0; i < HUDElements.params->table_columns - 1; i++)
             ImGui::TableNextColumn();
         ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
-        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width * 1.3, "%.1f ms", 1000 / HUDElements.sw_stats->fps);
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width * 1.3, "min: %.1fms, max: %.1fms", min_frametime, max_frametime);
         ImGui::PopFont();
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         char hash[40];
@@ -475,18 +578,31 @@ void HudElements::frame_timing(){
                                 NULL, min_time, max_time,
                                 ImVec2(ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns, 50));
         } else {
+#ifdef MANGOAPP
+        int width = ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns - 30;
+#else
+        int width = ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns;
+#endif
             ImGui::PlotLines(hash, get_time_stat, HUDElements.sw_stats,
                             ARRAY_SIZE(HUDElements.sw_stats->frames_stats), 0,
                             NULL, min_time, max_time,
-                            ImVec2(ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns, 50));
+                            ImVec2(width, 50));
+#ifdef MANGOAPP
+            ImGui::SameLine();
+            ImGui::PushFont(HUDElements.sw_stats->font1);
+            ImGui::Text("%.1fms", frametime / 1000.f);
+            ImGui::PopFont();
+#endif
         }
         ImGui::PopStyleColor();
-
     }
 }
 
 void HudElements::media_player(){
 #ifdef HAVE_DBUS
+    if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_media_player])
+        return;
+
     ImGui::TableNextRow(); ImGui::TableNextColumn();
     uint32_t f_idx = (HUDElements.sw_stats->n_frames - 1) % ARRAY_SIZE(HUDElements.sw_stats->frames_stats);
     uint64_t frame_timing = HUDElements.sw_stats->frames_stats[f_idx].stats[OVERLAY_PLOTS_frame_timing];
@@ -495,7 +611,7 @@ void HudElements::media_player(){
     ImGui::PushFont(&scaled_font);
     {
         std::lock_guard<std::mutex> lck(main_metadata.mtx);
-        render_mpris_metadata(*HUDElements.params, main_metadata, frame_timing, true);
+        render_mpris_metadata(*HUDElements.params, main_metadata, frame_timing);
     }
     ImGui::PopFont();
 #endif
@@ -517,11 +633,8 @@ void HudElements::resolution(){
 void HudElements::show_fps_limit(){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_show_fps_limit]){
         int fps = 0;
-        double frame_time = (double)fps_limit_stats.targetFrameTime.count()/1000000;
-        fps = (1 / frame_time) *1000;
-        if (frame_time == 0.0){
-            fps = 0;
-        }
+        if (fps_limit_stats.targetFrameTime.count())
+            fps = 1000000000 / fps_limit_stats.targetFrameTime.count();
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::TextColored(HUDElements.colors.engine, "%s","FPS limit");
@@ -534,7 +647,7 @@ void HudElements::show_fps_limit(){
 void HudElements::custom_text_center(){
     ImGui::TableNextRow(); ImGui::TableNextColumn();
     ImGui::PushFont(HUDElements.sw_stats->font1);
-    std::string value = HUDElements.ordered_functions[HUDElements.place].second;
+    const std::string& value = HUDElements.ordered_functions[HUDElements.place].second;
     center_text(value);
     ImGui::TextColored(HUDElements.colors.text, "%s",value.c_str());
     ImGui::NewLine();
@@ -544,13 +657,13 @@ void HudElements::custom_text_center(){
 void HudElements::custom_text(){
     ImGui::TableNextRow(); ImGui::TableNextColumn();
     ImGui::PushFont(HUDElements.sw_stats->font1);
-    std::string value = HUDElements.ordered_functions[HUDElements.place].second;
+    const std::string& value = HUDElements.ordered_functions[HUDElements.place].second;
     ImGui::TextColored(HUDElements.colors.text, "%s",value.c_str());
     ImGui::PopFont();
 }
 
 void HudElements::_exec(){
-    std::string value = HUDElements.ordered_functions[HUDElements.place].second;
+    //const std::string& value = HUDElements.ordered_functions[HUDElements.place].second;
     ImGui::PushFont(HUDElements.sw_stats->font1);
     ImGui::TableNextColumn();
     for (auto& item : HUDElements.exec_list){
@@ -583,6 +696,7 @@ void HudElements::vkbasalt(){
 }
 
 void HudElements::battery(){
+#ifdef __linux__
     if (Battery_Stats.batt_count > 0) {
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_battery]) {
             ImGui::TableNextRow(); ImGui::TableNextColumn();
@@ -616,41 +730,251 @@ void HudElements::battery(){
                 ImGui::PushFont(HUDElements.sw_stats->font1);
                 ImGui::Text("W");
                 ImGui::PopFont();
+                ImGui::TableNextRow(); ImGui::TableNextColumn();
+                float hours;
+                float minutes;
+                // float seconds;
+                minutes = std::modf(Battery_Stats.remaining_time, &hours);
+                minutes *= 60;
+                // seconds = std::modf(minutes, &minutes);
+                // seconds *= 60;
+                ImGui::PushFont(HUDElements.sw_stats->font1);
+                ImGui::TextColored(HUDElements.colors.text, "%s", "Remaining Time");
+                ImGui::PopFont();
+                ImGui::TableNextColumn(); ImGui::TableNextColumn();
+                // ImGui::TextColored(HUDElements.colors.text, "%02.0f:%02.0f:%02.0f", hours, minutes, seconds);
+                right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%02.0f:%02.0f", hours, minutes);
             }
+            else {
+                ImGui::TableNextColumn();
+                right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_PLUG);
+            }
+        }
+    }
+#endif
+}
+
+void HudElements::gamescope_fsr(){
+#ifdef MANGOAPP
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_fsr]) {
+        ImGui::TableNextRow(); ImGui::TableNextColumn();
+        string FSR_TEXT;
+        ImVec4 FSR_COLOR;
+        if (g_fsrUpscale){
+            FSR_TEXT = "ON";
+            FSR_COLOR = HUDElements.colors.fps_value_high;
+        } else {
+            FSR_TEXT = "OFF";
+            FSR_COLOR = HUDElements.colors.fps_value_low;
+        }
+
+        ImGui::TextColored(HUDElements.colors.engine, "%s", "FSR");
+        ImGui::TableNextColumn();
+        right_aligned_text(FSR_COLOR, HUDElements.ralign_width, "%s", FSR_TEXT.c_str());
+        ImGui::TableNextColumn();
+        if (g_fsrUpscale){
+            if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_hide_fsr_sharpness]) {
+                right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", g_fsrSharpness);
+                ImGui::SameLine(0,1.0f);
+                ImGui::PushFont(HUDElements.sw_stats->font1);
+                ImGui::Text("Sharp");
+                ImGui::PopFont();
+            }
+        }
+    }
+#endif
+}
+
+void HudElements::gamescope_frame_timing(){
+#ifdef MANGOAPP
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_debug]) {
+        ImGui::TableNextRow(); ImGui::TableNextColumn();
+        ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+        ImGui::PushFont(HUDElements.sw_stats->font1);
+        ImGui::TextColored(HUDElements.colors.engine, "%s", "App");
+        for (size_t i = 0; i < HUDElements.params->table_columns - 1; i++)
+            ImGui::TableNextColumn();
+        ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+        auto min = std::min_element(gamescope_debug_app.begin(), gamescope_debug_app.end());
+        auto max = std::max_element(gamescope_debug_app.begin(), gamescope_debug_app.end());
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width * 1.3, "min: %.1fms, max: %.1fms", min[0], max[0]);
+        ImGui::PopFont();
+        ImGui::TableNextRow(); ImGui::TableNextColumn();
+        char hash[40];
+        snprintf(hash, sizeof(hash), "##%s", overlay_param_names[OVERLAY_PARAM_ENABLED_frame_timing]);
+        HUDElements.sw_stats->stat_selector = OVERLAY_PLOTS_frame_timing;
+        HUDElements.sw_stats->time_dividor = 1000000.0f; /* ns -> ms */
+        double min_time = 0.0f;
+        double max_time = 50.0f;
+
+        ImGui::PushStyleColor(ImGuiCol_FrameBg, ImVec4(0.0f, 0.0f, 0.0f, 0.0f));
+        ImGui::PlotLines("", gamescope_debug_app.data(),
+                gamescope_debug_app.size(), 0,
+                NULL, min_time, max_time,
+                    ImVec2(ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns - 30, 50));
+            ImGui::SameLine();
+            ImGui::PushFont(HUDElements.sw_stats->font1);
+            ImGui::Text("%.1fms", gamescope_debug_app.back());
+            ImGui::PopFont();
+        ImGui::PopStyleColor();
+        if (gamescope_debug_latency.back() > -1){
+            ImGui::TableNextRow(); ImGui::TableNextColumn();
+            ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+            ImGui::PushFont(HUDElements.sw_stats->font1);
+            ImGui::TextColored(HUDElements.colors.engine, "%s", "Latency");
+            for (size_t i = 0; i < HUDElements.params->table_columns - 1; i++)
+                ImGui::TableNextColumn();
+            ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
+            min = std::min_element(gamescope_debug_latency.begin(), gamescope_debug_latency.end());
+            max = std::max_element(gamescope_debug_latency.begin(), gamescope_debug_latency.end());
+            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width * 1.3, "min: %.1fms, max: %.1fms", min[0], max[0]);
+            ImGui::PopFont();
+            ImGui::TableNextRow(); ImGui::TableNextColumn();
+
+            ImGui::PushStyleColor(ImGuiCol_FrameBg, ImVec4(0.0f, 0.0f, 0.0f, 0.0f));
+            ImGui::PushStyleColor(ImGuiCol_PlotLines, ImVec4(0,0,1,1));
+            ImGui::PlotLines("", gamescope_debug_latency.data(),
+                    gamescope_debug_latency.size(), 0,
+                    NULL, min_time, max_time,
+                    ImVec2(ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns - 30, 50));
+            ImGui::SameLine();
+            ImGui::PushFont(HUDElements.sw_stats->font1);
+            ImGui::Text("%.1fms", gamescope_debug_latency.back());
+            ImGui::PopFont();
+            ImGui::PopStyleColor(2);
+        }
+    }
+#endif
+}
+
+void HudElements::gamepad_battery()
+{
+#ifdef __linux__
+    gamepad_update();
+
+    if (gamepad_found) {
+        gamepad_info();
+        for (int i = 0; i < gamepad_count; i++) {
+            std::string battery = gamepad_data[i].battery;
+            std::string name = gamepad_data[i].name;
+            std::string battery_percent = gamepad_data[i].battery_percent;
+            bool report_percent = gamepad_data[i].report_percent;
+            bool charging = gamepad_data[i].is_charging;
+
+            ImGui::TableNextRow(); ImGui::TableNextColumn();
+            ImGui::PushFont(HUDElements.sw_stats->font1);
+            ImGui::TextColored(HUDElements.colors.engine, "%s", name.c_str());
+            ImGui::TableNextColumn();
+            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gamepad_battery_icon]) {
+                if (charging)
+                    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
+                else {
+                    if (battery == "Full")
+                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_FULL);
+                    else if (battery == "High")
+                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_THREE_QUARTERS);
+                    else if (battery == "Normal")
+                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_HALF);
+                    else if (battery == "Low")
+                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_QUARTER);
+                    else if (battery == "Unknown")
+                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
+                }
+            }
+            else {
+                if (charging)
+                    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
+                else if (report_percent) {
+                    right_aligned_text(HUDElements.colors.text,HUDElements.ralign_width, "%s", battery_percent.c_str());
+                    ImGui::SameLine(0,1.0f);
+                    ImGui::Text("%%");
+                }
+                else {
+                    if (battery == "Unknown")
+                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
+                    else
+                        right_aligned_text(HUDElements.colors.text,HUDElements.ralign_width, "%s", battery.c_str());
+                }
+            }
+            ImGui::PopFont();
+        }
+    }
+#endif
+}
+
+void HudElements::frame_count(){
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_frame_count]){
+        ImGui::TableNextRow();
+        ImGui::TableNextColumn();
+        ImGui::PushFont(HUDElements.sw_stats->font1);
+        ImGui::TextColored(HUDElements.colors.engine, "Frame Count");
+        ImGui::TableNextColumn();
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%llu", HUDElements.sw_stats->n_frames);
+        ImGui::PopFont();
+    }
+}
+
+void HudElements::fan(){
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_fan] && fan_speed != -1) {
+        ImGui::TableNextRow(); ImGui::TableNextColumn();
+        ImGui::TextColored(HUDElements.colors.engine, "%s", "FAN");
+        ImGui::TableNextColumn();
+        right_aligned_text(HUDElements.colors.text,HUDElements.ralign_width, "%i", fan_speed);
+        ImGui::SameLine(0, 1.0f);
+        ImGui::PushFont(HUDElements.sw_stats->font1);
+        ImGui::Text("RPM");
+        ImGui::PopFont();
+    }
+}
+
+void HudElements::throttling_status(){
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_throttling_status]){
+        if (gpu_info.is_power_throttled || gpu_info.is_current_throttled || gpu_info.is_temp_throttled || gpu_info.is_other_throttled){
+            ImGui::TableNextRow(); ImGui::TableNextColumn();
+            ImGui::TextColored(HUDElements.colors.engine, "%s", "Throttling");
+            ImGui::TableNextColumn();
+            ImGui::TableNextColumn();
+            if (gpu_info.is_power_throttled)
+                right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "Power");
+            if (gpu_info.is_current_throttled)
+                right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "Current");
+            if (gpu_info.is_temp_throttled)
+                right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "Temp");
+            if (gpu_info.is_other_throttled)
+                right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "Other");
         }
     }
 }
 
-
 void HudElements::graphs(){
     ImGui::TableNextRow(); ImGui::TableNextColumn();
     ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
-    std::string value = HUDElements.ordered_functions[HUDElements.place].second;
-    std::vector<float> arr(50, 0);
+    const std::string& value = HUDElements.ordered_functions[HUDElements.place].second;
+    assert(kMaxGraphEntries >= graph_data.size());
+    std::vector<float> arr(kMaxGraphEntries - graph_data.size());
 
     ImGui::PushFont(HUDElements.sw_stats->font1);
     if (value == "cpu_load"){
         for (auto& it : graph_data){
             arr.push_back(float(it.cpu_load));
-            arr.erase(arr.begin());
         }
-        HUDElements.max = 100; HUDElements.min = 0;
+        HUDElements.max = 100;
+        HUDElements.min = 0;
         ImGui::TextColored(HUDElements.colors.engine, "%s", "CPU Load");
     }
 
     if (value == "gpu_load"){
         for (auto& it : graph_data){
             arr.push_back(float(it.gpu_load));
-            arr.erase(arr.begin());
         }
-        HUDElements.max = 100; HUDElements.min = 0;
+        HUDElements.max = 100;
+        HUDElements.min = 0;
         ImGui::TextColored(HUDElements.colors.engine, "%s", "GPU Load");
     }
 
     if (value == "cpu_temp"){
         for (auto& it : graph_data){
             arr.push_back(float(it.cpu_temp));
-            arr.erase(arr.begin());
         }
         if (int(arr.back()) > HUDElements.cpu_temp_max)
             HUDElements.cpu_temp_max = arr.back();
@@ -663,7 +987,6 @@ void HudElements::graphs(){
     if (value == "gpu_temp"){
         for (auto& it : graph_data){
             arr.push_back(float(it.gpu_temp));
-            arr.erase(arr.begin());
         }
         if (int(arr.back()) > HUDElements.gpu_temp_max)
             HUDElements.gpu_temp_max = arr.back();
@@ -676,7 +999,6 @@ void HudElements::graphs(){
     if (value == "gpu_core_clock"){
         for (auto& it : graph_data){
             arr.push_back(float(it.gpu_core_clock));
-            arr.erase(arr.begin());
         }
         if (int(arr.back()) > HUDElements.gpu_core_max)
             HUDElements.gpu_core_max = arr.back();
@@ -689,7 +1011,6 @@ void HudElements::graphs(){
     if (value == "gpu_mem_clock"){
         for (auto& it : graph_data){
             arr.push_back(float(it.gpu_mem_clock));
-            arr.erase(arr.begin());
         }
         if (int(arr.back()) > HUDElements.gpu_mem_max)
             HUDElements.gpu_mem_max = arr.back();
@@ -702,20 +1023,18 @@ void HudElements::graphs(){
     if (value == "vram"){
         for (auto& it : graph_data){
             arr.push_back(float(it.gpu_vram_used));
-            arr.erase(arr.begin());
         }
 
         HUDElements.max = gpu_info.memoryTotal;
         HUDElements.min = 0;
         ImGui::TextColored(HUDElements.colors.engine, "%s", "VRAM");
     }
-#ifdef __gnu_linux__
+#ifdef __linux__
     if (value == "ram"){
         if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_ram])
             HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_ram] = true;
         for (auto& it : graph_data){
             arr.push_back(float(it.ram_used));
-            arr.erase(arr.begin());
         }
 
         HUDElements.max = memmax;
@@ -742,9 +1061,9 @@ void HudElements::graphs(){
     ImGui::PopStyleColor(1);
 }
 
-void HudElements::sort_elements(std::pair<std::string, std::string> option){
-    auto param = option.first;
-    auto value = option.second;
+void HudElements::sort_elements(const std::pair<std::string, std::string>& option){
+    const auto& param = option.first;
+    const auto& value = option.second;
 
     // Use this to always add to front of vector
     //ordered_functions.insert(ordered_functions.begin(),std::make_pair(param,value));
@@ -754,26 +1073,36 @@ void HudElements::sort_elements(std::pair<std::string, std::string> option){
     if (param == "gpu_stats")       { ordered_functions.push_back({gpu_stats, value});              }
     if (param == "cpu_stats")       { ordered_functions.push_back({cpu_stats, value});              }
     if (param == "core_load")       { ordered_functions.push_back({core_load, value});              }
+#ifndef MANGOAPP
     if (param == "io_stats")        { ordered_functions.push_back({io_stats, value});               }
+    if (param == "arch")            { ordered_functions.push_back({arch, value});                   }
+    if (param == "wine")            { ordered_functions.push_back({wine, value});                   }
+    if (param == "procmem")         { ordered_functions.push_back({procmem, value});                }
+    if (param == "gamemode")        { ordered_functions.push_back({gamemode, value});               }
+    if (param == "vkbasalt")        { ordered_functions.push_back({vkbasalt, value});               }
+    if (param == "engine_version")  { ordered_functions.push_back({engine_version, value});         }
+    if (param == "vulkan_driver")   { ordered_functions.push_back({vulkan_driver, value});          }
+    if (param == "resolution")      { ordered_functions.push_back({resolution, value});             }
+    if (param == "show_fps_limit")  { ordered_functions.push_back({show_fps_limit, value});         }
+#endif
     if (param == "vram")            { ordered_functions.push_back({vram, value});                   }
     if (param == "ram")             { ordered_functions.push_back({ram, value});                    }
     if (param == "fps")             { ordered_functions.push_back({fps, value});                    }
-    if (param == "engine_version")  { ordered_functions.push_back({engine_version, value});         }
     if (param == "gpu_name")        { ordered_functions.push_back({gpu_name, value});               }
-    if (param == "vulkan_driver")   { ordered_functions.push_back({vulkan_driver, value});          }
-    if (param == "arch")            { ordered_functions.push_back({arch, value});                   }
-    if (param == "wine")            { ordered_functions.push_back({wine, value});                   }
     if (param == "frame_timing")    { ordered_functions.push_back({frame_timing, value});           }
     if (param == "media_player")    { ordered_functions.push_back({media_player, value});           }
-    if (param == "resolution")      { ordered_functions.push_back({resolution, value});             }
-    if (param == "show_fps_limit")  { ordered_functions.push_back({show_fps_limit, value});         }
     if (param == "custom_text")     { ordered_functions.push_back({custom_text, value});            }
     if (param == "custom_text_center")  { ordered_functions.push_back({custom_text_center, value}); }
-    if (param == "gamemode")        { ordered_functions.push_back({gamemode, value});               }
-    if (param == "vkbasalt")        { ordered_functions.push_back({vkbasalt, value});               }
     if (param == "exec")            { ordered_functions.push_back({_exec, value});
                                       exec_list.push_back({int(ordered_functions.size() - 1), value});       }
     if (param == "battery")         { ordered_functions.push_back({battery, value});                }
+    if (param == "fps_only")        { ordered_functions.push_back({fps_only, value});               }
+    if (param == "fsr")             { ordered_functions.push_back({gamescope_fsr, value});          }
+    if (param == "debug")           { ordered_functions.push_back({gamescope_frame_timing, value}); }
+    if (param == "gamepad_battery") { ordered_functions.push_back({gamepad_battery, value});        }
+    if (param == "frame_count")     { ordered_functions.push_back({frame_count, value});            }
+    if (param == "fan")             { ordered_functions.push_back({fan, value});                    }
+    if (param == "throttling_status")        { ordered_functions.push_back({throttling_status, value});               }
     if (param == "graphs"){
         if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_graphs])
             HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_graphs] = true;
@@ -782,7 +1111,9 @@ void HudElements::sort_elements(std::pair<std::string, std::string> option){
             if (find(permitted_params.begin(), permitted_params.end(), value) != permitted_params.end())
                 ordered_functions.push_back({graphs, value});
             else
-                printf("MANGOHUD: Unrecognized graph type: %s\n", value.c_str());
+            {
+                spdlog::error("Unrecognized graph type: {}", value);
+            }
         }
     }
     return;
@@ -796,21 +1127,36 @@ void HudElements::legacy_elements(){
     ordered_functions.push_back({gpu_stats,          value});
     ordered_functions.push_back({cpu_stats,          value});
     ordered_functions.push_back({core_load,          value});
+#ifndef MANGOAPP
     ordered_functions.push_back({io_stats,           value});
+#endif
     ordered_functions.push_back({vram,               value});
     ordered_functions.push_back({ram,                value});
     ordered_functions.push_back({battery,            value});
+    ordered_functions.push_back({fan,                value});
+    ordered_functions.push_back({gamescope_fsr,      value});
+    ordered_functions.push_back({throttling_status,  value});
     ordered_functions.push_back({fps,                value});
+    ordered_functions.push_back({fps_only,           value});
+#ifndef MANGOAPP
     ordered_functions.push_back({engine_version,     value});
+#endif
     ordered_functions.push_back({gpu_name,           value});
+#ifndef MANGOAPP
     ordered_functions.push_back({vulkan_driver,      value});
     ordered_functions.push_back({arch,               value});
     ordered_functions.push_back({wine,               value});
+#endif
     ordered_functions.push_back({frame_timing,       value});
+    ordered_functions.push_back({frame_count,         value});
+    ordered_functions.push_back({gamescope_frame_timing, value});
+#ifndef MANGOAPP
     ordered_functions.push_back({gamemode,           value});
     ordered_functions.push_back({vkbasalt,           value});
     ordered_functions.push_back({show_fps_limit,     value});
     ordered_functions.push_back({resolution,         value});
+#endif
+    ordered_functions.push_back({gamepad_battery,    value});
     ordered_functions.push_back({media_player,       value});
 }
 
