@@ -1,9 +1,11 @@
 #include <sstream>
 #include <iomanip>
+#include <spdlog/spdlog.h>
 #include "logging.h"
 #include "overlay.h"
 #include "config.h"
 #include "file_utils.h"
+#include "string_utils.h"
 
 string os, cpu, gpu, ram, kernel, driver, cpusched;
 bool sysInfoFetched = false;
@@ -14,27 +16,20 @@ logData currentLogData = {};
 std::unique_ptr<Logger> logger;
 
 string exec(string command) {
-   char buffer[128];
-   string result = "";
-#ifdef __gnu_linux__
-
-   // Open pipe to file
-   FILE* pipe = popen(command.c_str(), "r");
-   if (!pipe) {
-      return "popen failed!";
-   }
-
-   // read till end of process:
-   while (!feof(pipe)) {
-
-      // use buffer to read and add to result
-      if (fgets(buffer, 128, pipe) != NULL)
-         result += buffer;
-   }
-
-   pclose(pipe);
+#ifndef _WIN32
+    if (getenv("LD_PRELOAD"))
+        unsetenv("LD_PRELOAD");
 #endif
-   return result;
+    std::array<char, 128> buffer;
+    std::string result;
+    std::unique_ptr<FILE, decltype(&pclose)> pipe(popen(command.c_str(), "r"), pclose);
+    if (!pipe) {
+      return "popen failed!";
+    }
+    while (fgets(buffer.data(), buffer.size(), pipe.get()) != nullptr) {
+      result += buffer.data();
+    }
+    return result;
 }
 
 void upload_file(std::string logFile){
@@ -56,11 +51,62 @@ void upload_files(const std::vector<std::string>& logFiles){
   exec("xdg-open " + url);
 }
 
+bool compareByFps(const logData &a, const logData &b)
+{
+    return a.fps < b.fps;
+}
+
+void writeSummary(string filename){
+  auto& logArray = logger->get_log_data();
+  filename = filename.substr(0, filename.size() - 4);
+  filename += "_summary.csv";
+  printf("%s\n", filename.c_str());
+  SPDLOG_DEBUG("Writing summary log file [{}]", filename, logArray.size());
+  std::ofstream out(filename, ios::out | ios::app);
+  if (out){
+    out << "0.1% Min FPS," << "1% Min FPS," << "97% Percentile FPS," << "Average FPS," << "GPU Load," << "CPU Load" << "\n";
+    std::vector<logData> sorted = logArray;
+    std::sort(sorted.begin(), sorted.end(), compareByFps);
+    float total = 0.0f;
+    float total_cpu = 0.0f;
+    float total_gpu = 0.0f;
+    float result;
+    float percents[2] = {0.001, 0.01};
+    for (auto percent : percents){
+      total = 0;
+      size_t idx = ceil(sorted.size() * percent);
+      for (size_t i = 0; i < idx; i++){
+        total = total + sorted[i].fps;
+      }
+      result = total / idx;
+      out << fixed << setprecision(1) << result << ",";
+    }
+    // 97th percentile
+    result = sorted.empty() ? 0.0f : sorted[floor(0.97 * (sorted.size() - 1))].fps;
+    out << fixed << setprecision(1) << result << ",";
+    // avg
+    total = 0;
+    for (auto input : sorted){
+      total = total + input.fps;
+      total_cpu = total_cpu + input.cpu_load;
+      total_gpu = total_gpu + input.gpu_load;
+    }
+    result = total / sorted.size();
+    out << fixed << setprecision(1) << result << ",";
+    // GPU
+    result = total_gpu / sorted.size();
+    out << result << ",";
+    // CPU
+    result = total_cpu / sorted.size();
+    out << result;
+  } else {
+    printf("MANGOHUD: Failed to write log file\n");
+  }
+}
+
 void writeFile(string filename){
   auto& logArray = logger->get_log_data();
-#ifndef NDEBUG
-  std::cerr << "Writing log file [" << filename << "], " << logArray.size() << " entries\n";
-#endif
+  SPDLOG_DEBUG("Writing log file [{}], {} entries", filename, logArray.size());
   std::ofstream out(filename, ios::out | ios::app);
   if (out){
   out << "os," << "cpu," << "gpu," << "ram," << "kernel," << "driver," << "cpuscheduler" << endl;
@@ -81,7 +127,6 @@ void writeFile(string filename){
     out << logArray[i].ram_used << ",";
     out << std::chrono::duration_cast<std::chrono::nanoseconds>(logArray[i].previous).count() << "\n";
   }
-  logger->clear_log_data();
   } else {
     printf("MANGOHUD: Failed to write log file\n");
   }
@@ -96,23 +141,13 @@ string get_log_suffix(){
   return log_name;
 }
 
-void logging(void *params_void){
-  overlay_params *params = reinterpret_cast<overlay_params *>(params_void);
-  logger->wait_until_data_valid();
-  while (logger->is_active()){
-      logger->try_log();
-      this_thread::sleep_for(chrono::milliseconds(params->log_interval));
-  }
-}
-
-Logger::Logger(overlay_params* in_params)
-  : m_logging_on(false),
-    m_values_valid(false),
-    m_params(in_params)
+Logger::Logger(const overlay_params* in_params)
+  : m_params(in_params),
+    m_logging_on(false),
+    m_values_valid(false)
 {
-#ifndef NDEBUG
-  std::cerr << "Logger constructed!\n";
-#endif
+  m_log_end = Clock::now() - 15s;
+  SPDLOG_DEBUG("Logger constructed!");
 }
 
 void Logger::start_logging() {
@@ -121,7 +156,7 @@ void Logger::start_logging() {
   m_logging_on = true;
   m_log_start = Clock::now();
   if((!m_params->output_folder.empty()) && (m_params->log_interval != 0)){
-    std::thread(logging, m_params).detach();
+    std::thread(&Logger::logging, this).detach();
   }
 }
 
@@ -130,14 +165,33 @@ void Logger::stop_logging() {
   m_logging_on = false;
   m_log_end = Clock::now();
 
-  std::thread(calculate_benchmark_data, m_params).detach();
+  calculate_benchmark_data();
 
   if(!m_params->output_folder.empty()) {
     std::string program = get_wine_exe_name();
     if (program.empty())
         program = get_program_name();
     m_log_files.emplace_back(m_params->output_folder + "/" + program + "_" + get_log_suffix());
-    std::thread(writeFile, m_log_files.back()).detach();
+    std::thread writefile (writeFile, m_log_files.back());
+    std::thread writesummary (writeSummary, m_log_files.back());
+    writefile.join();
+    writesummary.join();
+  } else {
+#ifdef MANGOAPP
+    string path = std::getenv("HOME");
+    std::string logName = path + "/mangoapp_" + get_log_suffix();
+    writeSummary(logName);
+    writeFile(logName);
+#endif
+  }
+  clear_log_data();
+}
+
+void Logger::logging(){
+  wait_until_data_valid();
+  while (is_active()){
+      try_log();
+      this_thread::sleep_for(chrono::milliseconds(m_params->log_interval));
   }
 }
 
@@ -148,7 +202,7 @@ void Logger::try_log() {
   auto elapsedLog = now - m_log_start;
 
   currentLogData.previous = elapsedLog;
-  currentLogData.fps = fps;
+  currentLogData.fps = 1000.f / (frametime / 1000.f);
   currentLogData.frametime = frametime;
   m_log_array.push_back(currentLogData);
 
@@ -182,3 +236,59 @@ void autostart_log(int sleep) {
   os_time_sleep(sleep * 1000000);
   logger->start_logging();
 }
+
+void Logger::calculate_benchmark_data(){
+  vector<float> sorted = benchmark.fps_data;
+  std::sort(sorted.begin(), sorted.end());
+  benchmark.percentile_data.clear();
+
+  benchmark.total = 0.f;
+  for (auto fps_ : sorted){
+    benchmark.total = benchmark.total + fps_;
+  }
+
+  size_t max_label_size = 0;
+
+  float result;
+  for (std::string percentile : m_params->benchmark_percentiles) {
+      // special case handling for a mean-based average
+      if (percentile == "AVG") {
+        result = benchmark.total / sorted.size();
+      } else {
+        // the percentiles are already validated when they're parsed from the config.
+        float fraction = parse_float(percentile) / 100;
+
+        result = sorted.empty() ? 0.0f : sorted[(fraction * sorted.size()) - 1];
+        percentile += "%";
+      }
+
+      if (percentile.length() > max_label_size)
+        max_label_size = percentile.length();
+
+      benchmark.percentile_data.push_back({percentile, result});
+  }
+  string label;
+  float mins[2] = {0.01f, 0.001f}, total;
+  for (auto percent : mins){
+    total = 0;
+    size_t idx = ceil(sorted.size() * percent);
+    for (size_t i = 0; i < idx; i++){
+      total = total + sorted[i];
+    }
+    result = total / idx;
+
+    if (percent == 0.001f)
+      label = "0.1%";
+    if (percent == 0.01f)
+      label = "1%";
+
+    if (label.length() > max_label_size)
+      max_label_size = label.length();
+
+    benchmark.percentile_data.push_back({label, result});
+  }
+
+   for (auto& entry : benchmark.percentile_data) {
+      entry.first.append(max_label_size - entry.first.length(), ' ');
+   }
+}
